////////////////////////////////////////////////////////////////////////////////
//
//	Interface with environment and filesystem
//
//	TODO : improvements
//
//	Create a new L1 filename, with a given version number (V00..V99)
//
//	char *	Set_L1_filename (date, key, ext, version) :
//
//	Search for latest version available (using glob) or NULL if file doesn't exists
//
//	char * 	Get_L1_filename (date, key, ext, version) :
//
////////////////////////////////////////////////////////////////////////////////

#define	MODULE_NAME	"INTERFACE"

#include "TOOLS.h"
#include "INTERFACE.h"

static	char *	solar_root = NULL;

static	glob_t	buffer;			// buffer data used bu glob()


////////////////////////////////////////////////////////////////////////////////
//
//	Check SOLAR_ROOT environment variable
//
t_err	Check_SOLAR_root (void)
{
	char *		fname = FNAME ("Check_SOLAR_root");
	t_err		error = OK;

	if ((solar_root = getenv ("SOLAR_ROOT")) == NULL) {

		Error_log (fname, "Variable SOLAR_ROOT not set");
		error = ERROR;
		goto EXIT;
	}
EXIT:	return error;
}


////////////////////////////////////////////////////////////////////////////////
//
//	Get SOLAR root directory
//
char *	Get_SOLAR_root ()
{
	return	solar_root;
}


////////////////////////////////////////////////////////////////////////////////
//
//	Read line in an Ascii file
//
t_err	Read_line (FILE * input, int size, char * buffer)
{
	char *		fname = FNAME ("Read_line");
	t_err		error = OK;
	int		end;

	if (fgets (buffer, size, input) == NULL) {

		strcpy (buffer, "");
		error = feof (input) ? WAR_eof : ERR_read;
		goto EXIT;
	}

	end = strlen (buffer) -1;

	if (end >= 0 && buffer [end] == '\n') buffer [end] = 0;

	end --;

	if (end >= 0 && buffer [end] == '\r') buffer [end] = 0;
	
EXIT:	return error;
}


////////////////////////////////////////////////////////////////////////////////
//
//	Search filenames corresponding to pattern
//
t_err	Search_files (char * pattern, int * count)
{
	char *		fname = FNAME ("Search_files");
	t_err		error = OK;

	* count = 0;

	buffer.gl_offs = 0;

	if (glob (pattern, 0, NULL, & buffer) == -1) {

		Error_log (fname, "Erreur glob () : pattern = %s : ", pattern);
		error = ERR_glob;
		goto EXIT;
	}

	* count = buffer.gl_pathc;

EXIT:	return error;	
}


////////////////////////////////////////////////////////////////////////////////
//
//	Free glob() ressources
//
void	Search_free (void)
{
	globfree (& buffer);

	buffer.gl_pathc = 0;
}


////////////////////////////////////////////////////////////////////////////////
//
//	Return nth filename 
//
char *	Get_filename (int number)
{
	if (number < 0 || number >= buffer.gl_pathc) return (char *) NULL;

	return buffer.gl_pathv [number];
}


////////////////////////////////////////////////////////////////////////////////
//
//	Check if file exists
//
t_err	Check_file_exists (char * filename)
{
	char *		fname = "Check_file_exists";
	t_err		error = OK;

	if (access (filename, F_OK) == -1) {

		Error_log (fname, "Unable to open %s", filename);
		error = ERROR;
		goto EXIT;
	}

EXIT:	return error;
}


////////////////////////////////////////////////////////////////////////////////
//
//	Create directory if it not exists
//
t_err	Create_directory (t_filename directory)
{
	char *		fname = FNAME ("Create_directory");
	t_err		error = OK;
	int		status;

	if ((status = mkdir (directory, 0777)) == -1) {

		if (errno != EEXIST) {

			Error_log (fname, "mkdir %s failed", directory);
			error = ERROR;
			goto EXIT;
		}
	}
	else	Message_log (3, fname, "Creating directory %s", directory);

EXIT:	return error;
}


////////////////////////////////////////////////////////////////////////////////
//
//	Return location of master CDF files
//
char *	Get_master_CDF_directory ()
{
	char *			fname = FNAME ("Get_master_CDF_directory");
	t_err			error = OK;
	static t_filename	directory;

	sprintf (directory, "%s/SKELETONS", solar_root);

	return directory;
}


////////////////////////////////////////////////////////////////////////////////
//
//	Compute L0 directory for a given spacecraft and date
//
char *	Get_L0_directory (t_date date)
{
	char *			fname = FNAME ("Get_L0_directory");
	t_err			error = OK;
	static t_filename	directory;

	sprintf (directory, "%s/DATA/L0/%s", solar_root, yyyymmdd (date));

	return directory;
}


////////////////////////////////////////////////////////////////////////////////
//
//	Compute L1 directory for a given date
//
char *	Get_L1_directory (t_date date)
{
	char *			fname = FNAME ("Get_L1_directory");
	t_err			error = OK;
	static t_filename	directory;

	sprintf (directory, "%s/DATA/L1/%s", solar_root, yyyymmdd (date));

	return directory;
}


////////////////////////////////////////////////////////////////////////////////
//
//	Compute L2 directory for a given date
//
char *	Get_L2_directory (t_date date)
{
	char *			fname = FNAME ("Get_L2_directory");
	t_err			error = OK;
	static t_filename	directory;

	sprintf (directory, "%s/DATA/L2/%s", solar_root, yyyymmdd (date));

	return directory;
}


////////////////////////////////////////////////////////////////////////////////
//
//	Create new L0 filename for a given date, key and extension 
//
//	Force version number (V00)
//
//	${SOLAR_ROOT}/DATA/L0/yyyymmdd/solo_L1_<key>_<yyyymmdd>_V<version>.<ext>
//
char *	New_L0_filename (t_date date, char * key, char * ext)
{
	char *			fname = FNAME ("New_L0_filename");
	t_err			error = OK;
	static t_filename	filename;
	int			version	= 0;

	sprintf (filename, "%s/solo_L0_%s_%s_V%02d.%s", 
		Get_L0_directory (date),
		key,
		yyyymmdd (date),
		version,
		ext);

	return filename;
}


////////////////////////////////////////////////////////////////////////////////
//
//	Create new L1 filename for a given date, key and extension 
//
//	Force version number (V00)
//
//	${SOLAR_ROOT}/DATA/L1/yyyymmdd/solo_L1_<key>_<yyyymmdd>_V<version>.<ext>
//
char *	New_L1_filename (t_date date, char * key, char * ext)
{
	char *			fname = FNAME ("New_L1_filename");
	t_err			error = OK;
	static t_filename	filename;
	int			version	= 0;

	sprintf (filename, "%s/solo_L1_%s_%s_V%02d.%s", 
		Get_L1_directory (date),
		key,
		yyyymmdd (date),
		version,
		ext);

	return filename;
}


////////////////////////////////////////////////////////////////////////////////
//
//	Create new L2 filename for a given date, key and extension 
//
//	Force version number (V00)
//
//	${SOLAR_ROOT}/DATA/L2/yyyymmdd/solo_L2_<key>_<yyyymmdd>_V<version>.<ext>
//
char *	New_L2_filename (t_date date, char * key, char * ext)
{
	char *			fname = FNAME ("New_L2_filename");
	t_err			error = OK;
	static t_filename	filename;
	int			version	= 0;

	sprintf (filename, "%s/solo_L2_%s_%s_V%02d.%s", 
		Get_L2_directory (date),
		key,
		yyyymmdd (date),
		version,
		ext);

	return filename;
}

////////////////////////////////////////////////////////////////////////////////
//
//	Search L0 filename for a given date, key and extension
//
//	Retrieve latest version available
//
//	${SOLAR_ROOT}/DATA/L0/yyyymmdd/solo_L0_<key>_<yyyymmdd>_V*.<ext>
//
char *	Get_L0_filename (t_date date, char * key, char * ext)
{
	char *			fname = FNAME ("Get_L0_filename");
	t_err			error = OK;
	t_filename		pattern;
	static t_filename	filename;
	int			count;
	
	strcpy (filename, "");

	sprintf (pattern, "%s/solo_L0_%s_%s_V*.%s",
		Get_L0_directory (date),
		key,
		yyyymmdd (date),
		ext);

	if (Error (error = Search_files (pattern, & count))) goto EXIT;

	if (count > 0) 
		strcpy (filename, Get_filename (count -1));
	else
		Warning_log (fname, "No file found : %s", pattern);

EXIT:	Search_free();

	return filename;
}	


////////////////////////////////////////////////////////////////////////////////
//
//	Return L1 filename for a given instrument and key
//
//	$SOLAR_ROOT/DATA/L1/yyyymmdd/solo_L1_swa-pas-<key>_<yyyymmdd>_Vxx.<ext>
//
char *	Get_L1_filename (t_date date, char * key, char * ext)
{
	char *			fname = FNAME ("Get_L1_filename");
	t_err			error = OK;
	t_filename		pattern;
	static t_filename	filename;
	int			count;
	
	strcpy (filename, "");

	sprintf (pattern, "%s/solo_L1_%s_%s_V*.%s",
		Get_L1_directory (date),
		key,
		yyyymmdd (date),
		ext);

	if (Error (error = Search_files (pattern, & count))) goto EXIT;

	if (count > 0) 
		strcpy (filename, Get_filename (count -1));
	else
		Warning_log (fname, "No file found : %s", pattern);

EXIT:	Search_free();

	return filename;
}	


////////////////////////////////////////////////////////////////////////////////
//
//	Return L2 filename for a given date and key
//
//	$SOLAR_ROOT/DATA/L1/yyyymmdd/solo_L2_swa-pas-<key>_<yyyymmdd>_Vxx.<ext>
//
char *	Get_L2_filename (t_date date, char * key, char * ext)
{
	char *			fname = FNAME ("Get_L2_filename");
	t_err			error = OK;
	t_filename		pattern;
	static t_filename	filename;
	int			count;
	
	strcpy (filename, "");

	sprintf (pattern, "%s/solo_L2_%s_%s_V*.%s",
		Get_L2_directory (date),
		key,
		yyyymmdd (date),
		ext);

	if (Error (error = Search_files (pattern, & count))) goto EXIT;

	if (count > 0) 
		strcpy (filename, Get_filename (count -1));
	else
		Warning_log (fname, "No file found : %s", pattern);

EXIT:	Search_free();

	return filename;
}	
