///////////////////////////////////////////////////////////////////////////////
//
//	Module	TMP
//
//	Handling temporary files
//
//	Files are generated in /DATA/SOLAR/DATA/L1/yyyymmdd.tmp
//	then copied finally in /DATA/SOLAR/DATA/L1/yyyymmdd
//
//	That allows to plot previous CDF files, while reprocessinf new ones in "real time"
//
////////////////////////////////////////////////////////////////////////////////

#define	MODULE_NAME	"TMP"

#include <glob.h>

#include "TMP.h"


////////////////////////////////////////////////////////////////////////////////
//
//	Return name of temporary directory for a given day 
//
//	/DATA/SOLAR/DATA/L1/yyyymmdd.tmp
//	
char *	Tmp_L1_directory (t_date date)
{
	static t_filename	directory;

	sprintf (directory, "%s.tmp", Get_L1_directory (date));

	return directory;
}


////////////////////////////////////////////////////////////////////////////////
//
//	Return temporary L1 filename
//
char *	Tmp_L1_filename (t_date date, char * key, char * ext)
{
	static t_filename	filename;

	sprintf (filename, "%s/%s", 
		Tmp_L1_directory (date), 
		basename (New_L1_filename (date, key, ext)));

	return filename;
}


////////////////////////////////////////////////////////////////////////////////
//
//	Move temporary files to final directory
//	
t_err	Move_tmp_files (t_date date)
{
	char *		fname = FNAME ("Move_tmp_files");
	t_err		error = OK;
	t_filename	pattern;
	int		done = FALSE;
	char *		src_dir = Tmp_L1_directory (date);
	char *		dst_dir = Get_L1_directory (date);
	int		retry = 0;
	int		count;

	sprintf (pattern, "%s/solo_L1_*", src_dir);

	Message_log (1, fname, "Move files to %s", dst_dir);

	// Try to move file from yyyymmdd.tmp to yyyymmdd (3 retry possibles)
	
	while (done == FALSE) {

		int	i;

		done = TRUE;

		if (Error (error = Search_files (pattern, & count))) goto EXIT;

		for (i = 0; i < count; i++) {

			t_filename	new;
			char *		old = Get_filename (i);
		
			sprintf (new, "%s/%s", dst_dir, basename (old));

			if (rename (old, new) == -1) {

				Error_log (fname, "Rename %s : err = %s", old, strerror (errno));
				done = FALSE;
			}
		}

		Search_free();

		if (done == FALSE)  {

			if (retry ++ > 3) break;

			Warning_log (fname, "Retry = %d", retry);
			usleep (500000);
		}
	}

	if (done == FALSE) {

		Error_log (fname, "Not all files have been moved to final directory");
		error = ERR_write;
		goto EXIT;
	}

	Message_log (1, fname, "Deleting %s", src_dir);

	if (rmdir (src_dir) == -1) {

		Error_log (fname, "Unable to delete %s directory", src_dir);
		error = ERROR;
		goto EXIT;
	}

EXIT:	return error;
}
