////////////////////////////////////////////////////////////////////////////////
//
//	Various tools shared by SOLAR softwares
//
////////////////////////////////////////////////////////////////////////////////

#define	MODULE_NAME	"TYPES"

#define	_XOPEN_SOURCE 
#include "TOOLS.h"


////////////////////////////////////////////////////////////////////////////////
//
//	Return localtime as a string YYYY-MM-DDThh:mm:ssZ
//	
char *	Now ()
{
	static char	str_time [30];
	struct tm *	now;
	time_t		seconds;

	seconds = time (NULL);

	now = localtime (& seconds);

	strftime (str_time, 30, "%FT%TZ", now);

	return str_time;
}


////////////////////////////////////////////////////////////////////////////////
//
//	Convert ISOTIME string to UTC time_t
//
//	This function use timegm() rather than mktime()
//	
//	timegm() interprets input struct tm as UTC
//	mktime() interprest input struct tm as localtime
//
time_t	str_to_time (char * str)
{
	struct tm	tm;

	memset (& tm, 0, sizeof (struct tm));

	if (strptime (str, "%Y-%m-%dT%T", & tm) == NULL) return -1;

	return timegm (& tm);
}


////////////////////////////////////////////////////////////////////////////////
//
//	Convert time_t to ISOTIME
//
//	WARNING : localtime 
//
char *	time_to_str (time_t t)
{
	static char	buffer [40];

	strftime (buffer, 40, "%FT%TZ", gmtime (& t));

	return buffer;
}

////////////////////////////////////////////////////////////////////////////////
//
//	Conversion string => date
//
t_err	str_to_date (const char * str, t_date * date)
{
	t_err		error = OK;
	int		y, m, d;

	y = m = d = 0;

	if (sscanf (str, "%d-%d-%d", & y, &m, &d) == 3) {

		date->year = y;
		date->month = m;
		date->day = d;
	}
	else	error = ERR_date;
	
	return error;
}


////////////////////////////////////////////////////////////////////////////////
//
//	Conversion date => string
//
char *	date_to_str (t_date date)
{
	static char	buffer [32];

	sprintf (buffer, "%04d-%02d-%02d", date.year, date.month, date.day);

	return buffer;
}


////////////////////////////////////////////////////////////////////////////////
//
//	Convert date to yyyymmdd string
//
char *	yyyymmdd (t_date date)
{
	static	char 	buffer [10];
	
	sprintf (buffer, "%04d%02d%02d", date.year, date.month, date.day);

	return buffer;
}


////////////////////////////////////////////////////////////////////////////////
//
//	Hexadecimal dump to string
//
char *	Hex_dump (unsigned char * ptr, int size)
{
	static	char	buffer [512];
	int		i;

	assert (3 * size  < sizeof (buffer));

	strcpy (buffer, "");

	for (i = 0; i < size; i++)
		sprintf (buffer, "%s%02X ", buffer, ptr [i]);

	return buffer;
}


////////////////////////////////////////////////////////////////////////////////
//
//	Return a 16 bits signed value from a starting address
//
int16_t get_int16 (byte_t * adr)
{
	return adr [0] << 8 | adr [1];
}


////////////////////////////////////////////////////////////////////////////////
//
//	Return a 16 bits value from a starting address
//
uint16_t get_uint16 (byte_t * adr)
{
	return adr [0] << 8 | adr [1];
}


////////////////////////////////////////////////////////////////////////////////
//
//	Return a 32 bits value from a starting address
//
uint32_t get_uint32 (byte_t * adr)
{
	return ((adr [0] << 8 | adr [1]) << 8 | adr [2]) << 8 | adr [3];
}


////////////////////////////////////////////////////////////////////////////////
//
//	Return a 64 bits value from a starting address
//
uint64_t get_uint64 (byte_t * adr)
{
	return ((((((adr [0] << 8 | adr [1]) << 8 | adr [2]) << 8 | adr [3]) << 8 | adr [4]) << 8 | adr [5]) << 8 | adr [6]) << 8 | adr [7];
}


////////////////////////////////////////////////////////////////////////////////
//
//	Extract Solar time stamps from byte buffer
//
t_err	get_time_stamp (t_so_time * time, byte_t * adr)
{
	char *	fname = FNAME ("get_time_stamp");
	t_err	error = OK;

	time->coarse = get_uint32 (adr);
	time->coarse &= 0x7FFFFFFF;		// Patch correction donnees 2018/12/24 => 2087/01/01
	time->fine   = get_uint16 (adr + 4);

	return error;
}


////////////////////////////////////////////////////////////////////////////////
//
//	Convert	binary
//
static
uint32_t	halfbits_to_floatbits (uint16_t h)
{
	uint16_t	h_exp, h_sig;
	uint32_t	f_sgn, f_exp, f_sig;
	h_exp = (h & 0x7c00u);

	f_sgn = ((uint32_t) h & 0x8000u) << 16;

	switch (h_exp) {

	case 0x0000u: /* 0 or subnormal */

		h_sig = (h&0x03ffu);

		/* Signed zero */
		if (h_sig == 0) {
			return f_sgn;
		}

		/* Subnormal */
		h_sig <<= 1;

		while ((h_sig&0x0400u) == 0) {
			h_sig <<= 1;
			h_exp++;
		}

		f_exp = ((uint32_t ) (127 - 15 - h_exp)) << 23;
		f_sig = ((uint32_t) (h_sig & 0x03ffu)) << 13;

		return f_sgn + f_exp + f_sig;

	case 0x7c00u: /* inf or NaN */
		/* All-ones exponent and a copy of the significand */
		return f_sgn + 0x7f800000u + (((uint32_t) (h & 0x03ffu)) << 13);

	default: /* normalized */
		/* Just need to adjust the exponent and shift */
		return f_sgn + (((uint32_t) (h & 0x7fffu) + 0x1c000u) << 13);
	}
}

////////////////////////////////////////////////////////////////////////////////
//
//	Convert from float16 to float
//
//	Source from npy_half_to_float
//
float	float16_to_float (uint16_t h)
{
	union {	float		ret;
		uint32_t	retbits; 
	}	conv;

	conv.retbits = halfbits_to_floatbits(h);

	return conv.ret;
}

